using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace TiledLevel
{
    class GameObject
    {
        public enum UpdateAction
        {
            None,
            Remove  // If an object returns this value from Update() it should
                    // be removed from the list of objects.
        }

        public enum HitAction
        {
            None,
            Damage
        }

        protected enum Flags
        {
            Delete = 0x0001     // The objects should be deleted
        }

        protected TextureContainer texture; // Texture used for the object, set by the derived class.

        protected Vector2 pos;                              // World position of the object.
        protected Vector2 size = new Vector2(1.0f, 1.0f);   // Size of the object.
        protected Vector2 vector = Vector2.Zero;            // Object's movement vector.
        protected float depth = 0.5f;                       // Determines when the object will be rendered.
                                                            // Objects are rendered from 1 to 0 so an object
                                                            // at depth 0.8 will be rendered before one at 0.5.
        protected Flags flags;

        /// <summary>
        /// Update the object's position and velocity.
        /// </summary>
        /// <param name="map">Map the object is on</param>
        public virtual UpdateAction Update(Map map, List<GameObject> objects)
        {
            if ((flags & Flags.Delete) != 0)
                return UpdateAction.Remove;

            return UpdateAction.None;
        }

        /// <summary>
        /// Render the object onto the given map.
        /// </summary>
        /// <param name="batch">SpriteBatch to use for rendering</param>
        /// <param name="map">Map the object should be drawn onto</param>
        public virtual void Draw(SpriteBatch batch, Map map)
        {
            map.DrawOnMap(batch, texture.tex, pos, depth);
        }

        /// <summary>
        /// Move the object's position based on the value of 'vector'.
        /// Object will be blocked by any walls.
        /// </summary>
        /// <param name="map">Map the object is moving on</param>
        /// <returns>Set of flags indicating the directions the object hit a wall</returns>
        protected virtual Direction Move(Map map)
        {
            Direction retval = Direction.None;
            Vector2 movement = vector;

            if (movement.X > 0.0f)
            {
                // We're moving to the right, create a bounding box in the area we're moving through.
                Vector2 collpos = new Vector2(pos.X + size.X, pos.Y);
                Vector2 collsize = new Vector2(movement.X, size.Y);
                float newx;

                // Check the area we want to move through against the map.
                if (map.Collide(collpos, collsize, Direction.Right, out newx))
                {
                    vector.X = 0f;
                    retval |= Direction.Right;
                }

                // Update the object's x position based on how far we could move.
                pos.X = newx - size.X;
            }
            else if (movement.X < 0.0f)
            {
                // We're moving to the left, create a bounding box in the area we're moving through.
                Vector2 collpos = new Vector2(pos.X + movement.X, pos.Y);
                Vector2 collsize = new Vector2(-movement.X, size.Y);
                float newx;

                // Check the area we want to move through against the map.
                if (map.Collide(collpos, collsize, Direction.Left, out newx))
                {
                    vector.X = 0f;
                    retval |= Direction.Left;
                }

                // Update the object's x position based on how far we could move.
                pos.X = newx;
            }

            if (movement.Y > 0.0f)
            {
                // We're moving down, create a bounding box in the area we're moving through.
                Vector2 collpos = new Vector2(pos.X, pos.Y + size.Y);
                Vector2 collsize = new Vector2(size.X, movement.Y);
                float newy;

                // Check the area we want to move through against the map.
                if (map.Collide(collpos, collsize, Direction.Down, out newy))
                {
                    vector.Y = 0f;
                    retval |= Direction.Down;
                }

                // Update the object's y position based on how far we could move.
                pos.Y = newy - size.Y;
            }
            else if (movement.Y < 0.0f)
            {
                // We're moving up, create a bounding box in the area we're moving through.
                Vector2 collpos = new Vector2(pos.X, pos.Y + movement.Y);
                Vector2 collsize = new Vector2(size.X, -movement.Y);
                float newy;

                // Check the area we want to move through against the map.
                if (map.Collide(collpos, collsize, Direction.Up, out newy))
                {
                    vector.Y = 0f;
                    retval |= Direction.Up;
                }

                // Update the object's y position based on how far we could move.
                pos.Y = newy;
            }

            // Return the set of flags indicating which directions we hit.
            return retval;
        }

        public virtual HitAction HitPlayer()
        {
            return HitAction.None;
        }

        public bool Hit(GameObject obj)
        {
            return (pos.X < obj.pos.X + obj.size.X &&
                    pos.Y < obj.pos.Y + obj.size.Y &&
                    pos.X + size.X > obj.pos.X &&
                    pos.Y + size.Y > obj.pos.Y);
        }
    }
}
