using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace TiledLevel
{
    class Map
    {
        private const int TILE_SIZE = 32;       // Width and height in pixels of each tile.

        private const UInt16 mapWidth = 64;     // Width of the map in tiles.
        public UInt16 Width
        {
            get { return mapWidth; }
        }

        private const UInt16 mapHeight = 64;    // Height of the map in tiles.
        public UInt16 Height
        {
            get { return mapHeight; }
        }

        // This array stores the textures that map to the ids stored in the grid.
        private static Texture2D[] tiles = new Texture2D[2];

        // This member stores which tile should be drawn in the centre of the screen.
        private Vector2 centre = new Vector2(8.0f, 56.0f);

        // This member stores the scale of the map.
        private float scale = 1.5f;

        // This is the 2D array that represents our map.  Only two values are used currently:
        //  0 - An empty tile
        //  1 - A brick tile
        private UInt16[,] grid;

        private SpriteBatch batch;

        public Map()
        {
            // Create the 2D grid which will store the map data.
            grid = new UInt16[mapWidth, mapHeight];

            // Create an edge round the map.
            for (int y = 0; y < mapHeight; y++)
            {
                grid[0, y] = 1;
                grid[mapWidth - 1, y] = 1;
            }
            for (int x = 0; x < mapWidth; x++)
            {
                grid[x, 0] = 1;
                grid[x, mapHeight - 1] = 1;
            }

            // Create a load of 6 tile wide platforms at random positions.
            for (int platform = 0; platform < 100; platform++)
            {
                int platx = Global.rnd.Next(mapWidth - 6);
                int platy = Global.rnd.Next(mapHeight);
                for (int tile = 0; tile < 6; tile++)
                    grid[platx + tile, platy] = 1;
            }

            // Create the spritebatch we will use for rendering
            batch = new SpriteBatch(Global.graphics.GraphicsDevice);
        }

        /// <summary>
        /// Render a texture onto the map using the given spritebatch and map position.
        /// </summary>
        /// <param name="batch">Spritebatch to render the texture with</param>
        /// <param name="tex">Texture we want to render</param>
        /// <param name="pos">Map position to render the texture at</param>
        public void DrawOnMap(SpriteBatch batch, Texture2D tex, Vector2 pos, float depth)
        {
            if (tex == null)
                return;

            // Find the screen position for the texture.
            Vector2 scrpos = WorldToScreen(pos);

            batch.Draw(tex, scrpos, null, Color.White, 0f, Vector2.Zero,
                       this.scale, SpriteEffects.None, depth);
        }

        /// <summary>
        /// Render the map to screen.
        /// </summary>
        public void Draw()
        {
            Vector2 topleft = new Vector2(0.0f, 0.0f);
            Vector2 bottomright = new Vector2(Global.screenWidth, Global.screenHeight);
            Vector2 toplefttile, bottomrighttile;
            int x, y;

            batch.Begin(SpriteBlendMode.None);

            // Find the tiles at the top-left and bottom-right of the screen by passing the
            // appropriate co-ordinates to the ScreenToWorld() method.
            toplefttile = ScreenToWorld(topleft);
            bottomrighttile = ScreenToWorld(bottomright);

            // Loop through the grid of tiles we know to be on the screen.
            for (y = (int)toplefttile.Y; y <= (int)bottomrighttile.Y; y++)
            {
                for (x = (int)toplefttile.X; x <= (int)bottomrighttile.X; x++)
                {
                    // Get the id of the tile at the current position.
                    UInt16 id = Grid(x, y);

                    // Render the correct texture at the current position.
                    DrawOnMap(batch, tiles[id], new Vector2((float)x, (float)y), 0f);
                }
            }

            batch.End();
        }

        /// <summary>
        /// Returns true if the given tile is solid
        /// </summary>
        public bool SolidSquare(int x, int y)
        {
            return (Grid(x, y) != 0);
        }

        /// <summary>
        /// Check if an object moving through the given space would have hit a wall.
        /// </summary>
        /// <param name="pos">Top-left co-ordinate of the movement area</param>
        /// <param name="size">Size of the movement area</param>
        /// <param name="dirn">Direction we want test moving through the area in</param>
        /// <param name="newpos">Return value indicating how far through the area an object can move</param>
        /// <returns>If an object would hit a wall moving through this area</returns>
        public bool Collide(Vector2 pos, Vector2 size, Direction dirn, out float newpos)
        {
            // Find the bounding box for the given area as whole tiles.
            float maxxf = (pos.X + size.X);
            float maxyf = (pos.Y + size.Y);
            int minx = (int)pos.X;
            int miny = (int)pos.Y;
            int maxx = (int)maxxf;
            int maxy = (int)maxyf;

            if (Global.IsInt(maxxf)) maxx--;
            if (Global.IsInt(maxyf)) maxy--;

            if (dirn == Direction.Right)
            {
                // We are travelling right, we will reach the right edge
                // of the given area if we don't hit anything.
                newpos = pos.X + size.X;

                // Scan through the tiles we're travelling through from left to right.
                for (int x = minx; x <= maxx; x ++)
                {
                    for (int y = miny; y <= maxy; y++)
                    {
                        if (SolidSquare(x, y))
                        {
                            // If we find a wall then return the position of the left
                            // hand side of it.
                            newpos = (float)x;
                            return true;
                        }
                    }
                }
                return false;
            }

            if (dirn == Direction.Left)
            {
                // We are travelling left, we will reach the left edge
                // of the given area if we don't hit anything.
                newpos = pos.X;
                
                // Scan through the tiles we're travelling through from right to left.
                for (int x = maxx; x >= minx; x--)
                {
                    for (int y = miny; y <= maxy; y++)
                    {
                        if (SolidSquare(x, y))
                        {
                            // If we find a wall then return the position of the right
                            // hand side of it.
                            newpos = (float)(x + 1);
                            return true;
                        }
                    }
                }
                return false;
            }

            if (dirn == Direction.Down)
            {
                // We are travelling down, we will reach the bottom edge
                // of the given area if we don't hit anything.
                newpos = pos.Y + size.Y;

                // Scan through the tiles we're travelling through from top to bottom.
                for (int y = miny; y <= maxy; y++)
                {
                    for (int x = minx; x <= maxx; x++)
                    {
                        if (SolidSquare(x, y))
                        {
                            // If we find a wall then return the position of the top of it.
                            newpos = (float)y;
                            return true;
                        }
                    }
                }
                return false;
            }

            if (dirn == Direction.Up)
            {
                // We are travelling up, we will reach the top edge
                // of the given area if we don't hit anything.
                newpos = pos.Y;

                // Scan through the tiles we're travelling through from bottom to top.
                for (int y = maxy; y >= miny; y--)
                {
                    for (int x = minx; x <= maxx; x++)
                    {
                        if (SolidSquare(x, y))
                        {
                            // If we find a wall then return the position of the bottom of it.
                            newpos = (float)(y + 1);
                            return true;
                        }
                    }
                }
                return false;
            }

            // Should shouldn't ever reach here but the compiler moans if we don't do this.
            newpos = 0.0f;
            return true;
        }

        /// <summary>
        /// Set the world co-ordinate which should be rendered in the centre of the screen.
        /// </summary>
        /// <param name="pos">World co-ordinate to centre screen on.</param>
        public void SetViewPos(Vector2 pos)
        {
            // Only move towards the specified position, don't move directly there.
            centre.X += (pos.X - centre.X) * 0.1f;
            centre.Y += (pos.Y - centre.Y) * 0.1f;
        }

        /// <summary>
        /// Returns the screen position matching the given map position.
        /// </summary>
        private Vector2 WorldToScreen(Vector2 pos)
        {
            pos -= centre;
            pos.X *= scale * TILE_SIZE;
            pos.Y *= scale * TILE_SIZE;
            pos.X += Global.screenWidth / 2;
            pos.Y += Global.screenHeight / 2;
            return pos;
        }

        /// <summary>
        /// Returns the world position matching the given screen position.
        /// </summary>
        private Vector2 ScreenToWorld(Vector2 pos)
        {
            pos.X -= Global.screenWidth / 2;
            pos.Y -= Global.screenHeight / 2;
            pos.X /= scale * TILE_SIZE;
            pos.Y /= scale * TILE_SIZE;
            pos += centre;
            return pos;
        }

        /// <summary>
        /// Returns the tile at the given position.  We call this method rather than directly index
        /// the 'grid' array as it provides bounds checking against the edges of the map.
        /// </summary>
        private UInt16 Grid(int x, int y)
        {
            // Check co-ordinate exists on map.
            if (x >= 0 && x < mapWidth &&
                y >= 0 && y < mapHeight)
                return grid[x, y];

            // All tiles outside of the stored map area are empty.
            return 0;
        }

        public static void LoadContent(ContentManager content)
        {
            tiles[1] = content.Load<Texture2D>("gfx/brick");
        }
    }
}
